package org.apache.activemq.test;

import java.util.concurrent.Callable;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** 
 * Message consumer running as a separate thread, connecting to broker
 */
public class Consumer implements Callable {

    public Object init = new Object();
    protected String queueName = "";
    boolean isTopic = false;

    Logger log = null;

    public Consumer(String destName, boolean topic) {
        log = LoggerFactory.getLogger(this.getClass());
        this.isTopic = topic;
        this.queueName = destName;
    }

    /**
     * connect to broker and receive messages
     */
    public Boolean call() {
        Connection connection = null;
        Session session = null;
        MessageConsumer consumer = null;

        try {
            ActiveMQConnectionFactory amq = new ActiveMQConnectionFactory(AMQTemplateTest.BROKERURL);
            connection = amq.createConnection();

            connection.setExceptionListener(new javax.jms.ExceptionListener() {
                    public void onException(javax.jms.JMSException e) {
                        e.printStackTrace();
                    }
            });
            connection.start();

            // Create a Session
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

            // Create the destination (Topic or Queue)
            Destination  destination = null;
            if (isTopic) 
                destination = session.createTopic(queueName);
            else 
                destination = session.createQueue(queueName);

            //Create a MessageConsumer from the Session to the Topic or Queue
            consumer = session.createConsumer(destination);
            // consumer = session.createDurableSubscriber((Topic)destination, "consumer1");

            long counter = 0;
            log.info("Consumer entering wait loop for messages");

            // Wait for a message
            for (int i = 0; i < AMQTemplateTest.NUM_MSGS; i++) {
                Message message2 = consumer.receive(10*1000);

                if (message2 == null) {
                    log.error("Did not receive a message");
                    return new Boolean(false);
                }
                if (message2 instanceof TextMessage) {
                    TextMessage textMessage = (TextMessage) message2;
                    String text = textMessage.getText();
                    log.debug("Received: " + text.substring(0, 50));
                } else {
                    log.warn("Received message of unsupported type. Expecting TextMessage. " + message2); 	     
                }
                counter ++;
                if ((counter % 1000) == 0)
                    log.info("received " + counter + " messages");
            }
            return new Boolean(true);
        } catch(Exception e) {
            log.error("Error in Consumer: " + e.getMessage());
            return new Boolean(false);
        }
        finally {
            try {
                if (consumer != null)
                    consumer.close();
                if (session != null)
                    session.close();
                if (connection != null)
                    connection.close();
            } catch (Exception ex) {
                log.error("Error closing down JMS objects: " + ex.getMessage());
            }
        }
    }
}
