package org.apache.activemq.test;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Message producer running as a separate thread, connecting to broker
 */
public class Producer implements Runnable{

    Logger log = null;
    protected int timeToLive=60000;
    protected String destName = "";


    public Producer(String dest, int ttl) {
        log = LoggerFactory.getLogger(this.getClass());
        this.destName = dest;
        this.timeToLive = ttl;
    }


    /**
     * Connect to broker and constantly send messages
     */
    public void run() {

        Connection connection = null;
        Session session = null;
        MessageProducer producer = null;

        try {
            ActiveMQConnectionFactory amq = new ActiveMQConnectionFactory(AMQTemplateTest.BROKERURL);
            connection = amq.createConnection();

            connection.setExceptionListener(new javax.jms.ExceptionListener() { 
                    public void onException(javax.jms.JMSException e) {
                        e.printStackTrace();
                    }
            });

            connection.start();

            // Create a Session
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

            // Create the destination (Topic or Queue)
            Destination destination = session.createTopic(destName);

            // Create a MessageProducer from the Session to the Topic or Queue
            producer = session.createProducer(destination);
            producer.setTimeToLive(timeToLive);

            // Create message
            long counter = 0;
            //enlarge msg to 10 KB
            int msgSize = 1024 * 10;
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.setLength(msgSize + 15);
            stringBuilder.append("Message: ");
            stringBuilder.append(counter);

            /*
            //enlarge msg to 10 KB
            for (int j = 0; j < (msgSize/10); j++) {
                stringBuilder.append("XXXXXXXXXX");
            }
            */
            String text = stringBuilder.toString();
            TextMessage message = session.createTextMessage(text);

            // send message
            for (int i = 0; i < AMQTemplateTest.NUM_MSGS; i++) {
                log.debug("Sent message: "+ counter);
                producer.send(message);
                counter ++;

                if ((counter % 1000) == 0)
                    log.info("sent " + counter + " messages");

            } 
        } catch (Exception ex) {
            log.error(ex.getMessage());
            return;
        }
        finally {
            try {
                if (producer != null) 
                    producer.close();
                if (session != null)
                    session.close();
                if (connection != null)
                    connection.close();
            } catch (Exception e) {
                log.error("Problem closing down JMS objects: " + e.getMessage());
            }
        }
        log.info("Closing producer for " + destName);
    }
}

