package org.apache.activemq.test;

import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;


import junit.framework.Assert;

import org.apache.activemq.transport.stomp.Stomp;
import org.apache.activemq.transport.stomp.StompConnection;
import org.apache.activemq.transport.stomp.StompFrame;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Message consumer running as a separate thread, connecting to broker
 * using stomp
 *
 */
public class StompConsumer implements Runnable{

    public Logger log = null;
    protected String queueName = "";
    protected boolean isTopic = false;
    protected long consumerSleep = 100;

    public StompConsumer(String destName, boolean topic, long sleep) {
        log = LoggerFactory.getLogger(this.getClass());
        this.queueName = destName;
        this.isTopic = topic;
        this.consumerSleep = sleep;
    }


    public void run() {

        log.info("Running consumer for " + queueName);
        StompConnection stompConnection = new StompConnection();
        int counter = 0;

        try {
            // open connection
            stompConnection.open("localhost", 61613);
            stompConnection.connect("admin", "admin");
            
            // stompConnection.sendFrame("CONNECT\n" + "login: admin\n" + "passcode: admin\n\n" + Stomp.NULL);
            // StompFrame frame = stompConnection.receive();
            // assertTrue(frame.toString().startsWith("CONNECTED"));

            HashMap<String, String> headers = new HashMap<String, String>();
            headers.put("activemq.prefetchSize", "2");
            log.debug("Going to subscribe to " + queueName);

            if (!isTopic)
                stompConnection.subscribe("/queue/" + queueName, "client", headers);
            else 
                stompConnection.subscribe("/topic/" + queueName, "client", headers);

            // receive all msgs
            StompFrame frame;
            for (counter = 0; counter < AMQTemplateTest.NUM_MSGS; counter++) {
                frame = stompConnection.receive(TimeUnit.SECONDS.toMillis(10));
                if (log.isDebugEnabled())
                    log.debug("Received msg with content: " + frame.getBody().substring(0, 20));
                stompConnection.ack(frame);

                if ((counter % 1000) == 0 && counter > 0)
                    log.info("received " + counter + " messages");
                Thread.sleep(consumerSleep);
            }

            /*
            // in addition check QueueSize using JMX
            JMXServiceURL url = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://:1099/jmxrmi");
            JMXConnector jmxc = JMXConnectorFactory.connect(url, null);
            MBeanServerConnection mbsc = jmxc.getMBeanServerConnection();
            Object tmp = mbsc.getAttribute(new ObjectName("org.apache.activemq:BrokerName=localhost,Type=Queue,Destination=Consumer.A.VirtualTopic.FOO"), "QueueSize");
            long queueSize = ((Long) tmp).longValue();
            */
        } catch (Exception ex) {
            log.error(ex.getMessage() + " after consuming " + counter + " msgs.", ex);
            
            // parent.setFail("Did only receive " + counter + " of " + StompVirtualTopicTest.NUM_MSGS +
            //  " messages. Check QueueSize in JMX and try to browse the queue.");
        } finally {
            try {
                stompConnection.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
        log.info("StompConsumer Finished.");
    }
}
