package org.apache.activemq.test;


import java.io.IOException;
import java.util.HashMap;

// import org.apache.activemq.transport.stomp.Stomp;
import org.apache.activemq.transport.stomp.StompConnection;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;


public class StompProducer implements Runnable {

    public Logger log = null;
    protected String destName = "";
    boolean isTopic = false;
    protected int numMsgs = 1;
    protected long producerSleep = 100;
    protected long ttl = -1;

    public StompProducer(int numMsgs, String destName, boolean topic, long sleep, long ttl) {
        log = LoggerFactory.getLogger(this.getClass());
        if (topic) {
            this.destName = "/topic/" + destName;
        } else {
            this.destName = "/queue/" + destName;
        }
        this.isTopic = topic;
        this.numMsgs = numMsgs;
        this.producerSleep = sleep;
        this.ttl = ttl;
    }


    public void run() {
        log.info("Running stomp producer for " + destName);
        StompConnection stompConnection = new StompConnection();
        int counter = 0;

        int msgSize = 1024*100;
        log.info("Generating 100K message");
        StringBuffer msgPayload = new StringBuffer(msgSize);
        for (int i=0;i<msgSize/10; i++) {
            msgPayload.append("xxxxxxxxxx");
        }
        
        try {
            // open connection
            stompConnection.open("localhost", 61613);
            stompConnection.connect("admin", "admin");
            
            // stompConnection.sendFrame("CONNECT\n" + "login: admin\n" + "passcode: admin\n\n" + Stomp.NULL);
            // StompFrame frame = stompConnection.receive();
            // assertTrue(frame.toString().startsWith("CONNECTED"));

            HashMap<String, String> headers = new HashMap<String, String>();
            //headers.put("activemq.prefetchSize", "2");
            //headers.put("expires", new Long(System.currentTimeMillis() + ttl).toString());
            headers.put("persistent", "true");
            
            log.debug("Going to send {}messages to {} ", numMsgs, destName);

            for (counter = 0; counter < numMsgs; counter++) {
                stompConnection.send(destName, msgPayload.toString(), null, headers);
                if (log.isDebugEnabled())
                    log.debug("Sent msg {} ", counter);
                
                if ((counter % 1000) == 0 && counter > 0)
                    log.info("Sent " + counter + " messages");
                Thread.sleep(producerSleep);
            }
        } catch (Exception ex) {
            log.error(ex.getMessage() + " while trying to sent msg # " + counter + ".", ex);
        } finally {
            try {
                stompConnection.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
        log.info("StompProducer finished.");
    }
}
